% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pwmean_s4_dispatch.R
\name{dispatch_estimator_one_domain}
\alias{dispatch_estimator_one_domain}
\title{Run the estimator for a single domain}
\usage{
dispatch_estimator_one_domain(build, Y, zvec, w, X)
}
\arguments{
\item{build}{A `pw_fit` object returned by the build step.}

\item{Y}{Outcome vector of length `n`.}

\item{zvec}{Domain indicator vector of length `n`.}

\item{w}{Pseudo-weight vector of length `n`.}

\item{X}{Design matrix for the convenience sample, with dimension
`n x p`.}
}
\value{
A list with components `mean` and `variance`.
}
\description{
Selects and calls the specific estimator function based on
`build$method`, passing the relevant matrices and vectors
for one domain at a time.
}
\keyword{internal}
